@@ -4,6 +4,7 @@
 
 use Lingua::EN::NameParse qw(&case_surname &clean);
 
+
 # Quick casing, no parsing or context check
 
 $input = "DE TANGELO";
@@ -73,9 +74,10 @@ MR AB AND M/S CD VAN DER HEIDEN-MACNAY
  MR AS & D.E. DE LA MARE
 ESTATE OF THE LATE AB LE FONTAIN
 BIG BROTHER & THE HOLDING COMPANY
-MR KA MACQUARIE
+MR KA MACQUARIE JNR.
 RIGHT HONOURABLE MR PJ KEATING
 MR TOM JONES
+MR JO TAYLOR IV
 EDWARD G WHITLAM
 JAMES BROWN
 MR AS SMI9TH
